<?php 
// +----------------------------------------------------------------------+
// | EMS NG Troubleshooting Checker scripts                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2013 VDL-5 Technologies Inc.                           |
// |                                                                      |
// +----------------------------------------------------------------------+
// | Authors: Tamara S Harpster <harpstert@vdl5tech.com                   |
// +----------------------------------------------------------------------+
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

include 'common.inc';
include 'config.inc';

// Set defaults for options in case they don't get set. Only exception is setting the file location for the replacement directives, have to have that.
$indir="C:\IETM\M915_IETM\T - Troubleshooting procedures\\";
$logfile="ts_check.log";
$ext=".xml";
$debug = false;
$all_files = array();
$clean_files = array();
$htmlFlag = false;
// Set up an Array to store the messages that are found during the checks
$report = array();
$print_test = true;
$print_parms = false;

if ($argc > 1) {
	// script is being run from command line so process data accordingly
	// Remember to finish up the File argument at some point in the future
	// Process the argument, set some defaults for directories if they don't get set. 
	for ($i=1;$i<$argc;$i++){
		switch ($argv[$i]) {
			case "-indir":
				$i++;
				$indir= (file_exists(addslashes($argv[$i]))) ? addslashes($argv[$i]) : "\\.";
				$all_files = scanDirectory($indir);
				$clean_files = str_ireplace($indir, "" , $all_files);
				break;
			case "-debug":
				$i++;
				$debug= addslashes($argv[$i]);
				break;
			case "-list":
				$i++;
				$fileList = (file_exists(addslashes($argv[$i]))) ? addslashes($argv[$i]) : "in_files.txt";
				$clean_files = file($fileList, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
				break;
			case "-log":
				$i++;
				if (!(file_exists(addslashes($argv[$i])))) {
					mkdir(addslashes($argv[$i]),0777, true);
				}
				$logfile= addslashes($argv[$i]);
				break;
			case "-file":
				$i++;
				print addslashes($argv[$i])."\r\n";
				if (file_exists(addslashes($argv[$i])) ){
					$lines_temp=file(addslashes($argv[$i]), FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
					$lines2_temp=implode($lines_temp);
					$lines = split($endToken,$lines2_temp);
					// added in a pop in order to clean off a blank being read in for the file, shrug, it works, and gets rids of the preg_replace errors so I'm happy
					$tmplines = array_pop($lines);
				} else {
					print "nope, sorry, no file";
					exit;
				}
				break;
			default:
		}
	}
} else {
	// calling this guy from a web page so set a few items here for use later on
	$htmlFlag = true;
	$clean_files = array_keys( $_GET, "on");
	if (isset($_GET['print_test'])) {
		$print_test = true;
	} else {
		$print_test = false;
	}
	if (isset($_GET['print_parms'])) {
		$print_parms = true;
	} else {
		$print_parms = false;
	}
	$indir = $base_dir.$ietms[$_GET['inputdir']]["dir"]."\\T - Troubleshooting procedures\\";
	$report[] = $header."Results of Check".$logoStart;
}
if ($htmlFlag) {
$report[]= "<p><strong>Current Directory:</strong> ".$indir."</p>";
	$fileList = "";
	foreach ($clean_files as $file_name){ 
		$fileList = $fileList."&".trim($file_name)."=on";
	}
	
	$report[] = "<ul>
					<li><a href='check_emsng.php?inputdir=".$_GET['inputdir'].$fileList."'>Click to check the same files</a></li>
					<li><a href='checker.php?inputdir=".$_GET['inputdir']."'>Click to check other files in this IETM</a></li>
					<li><a href='index.php'>Click to files in another IETM</a></li>
					</ul>";
}
// Dump out an index of the files being checked
if ($htmlFlag) {
	foreach ($clean_files as $file_name){ 
		$report[] = "<a href='#".trim($file_name)."'>".trim($file_name)." results</a><br>";
	}
}

foreach ($clean_files as $file_name){ 
// var_dump($file_name);
	$clean_text ="";
	if ($htmlFlag) {
		$clean_text = clean_xml($indir.trim($file_name).".xml");
	} else {
		$clean_text = clean_xml($indir.trim($file_name));
	}
		$report[] = "Check of file ".trim($file_name);
		if ($htmlFlag) {
			$report[] = "<h3><a name='".trim($file_name)."'>".array_pop($report)."</h3>";
		} 

	// do some initialization here of items that are needed internally to the program
	// Stores the links that have been dug out of the file along with results of various check on the links.
	$linksArray = array();
	$linksCntr = 0;
	// Keeps track of the testwithstate ids for the file so they can be used to check the file links
	$testIDArray = array();
	
	// Setup array to store internal links for checks against Testwithstates
	$intLinkArray = array();

	$wpno = "";

	$dom = new DomDocument();
	$dom->loadXML($clean_text);

	// dig out the diagnosticwp tag along with the wpno attribute and see if they match.
	// This is overkill for this but it matches in with the other DOM process
	if ($print_test) {
		$nodelist = $dom->getElementsByTagName('diagnosticwp');
		$wpno = "";
		if ($nodelist->length == 0){
		echo "no diagnosticwp to find";
		} else {
			for ($i = 0; $i < $nodelist->length; $i++) {
				$subfile =  substr($file_name,0, strrpos($file_name, '.')-1);
				// $subfile =  trim($file_name);
				$wpno = $nodelist->item($i)->getAttribute('wpno');
				// print ("wpno:".$wpno.":\r\nfile:".$subfile.":filename:".$file_name.":\r\n<br>");
				if (strcmp($file_name,$wpno) != 0){
					$report[] = "\t<h2>Warning: wpno and filename don\'t match</h2>";
				} else {
					$report[] =  "\tattribute wpno and filename match";
				}
				if (strlen($nodelist->item($i)->getAttribute('comment')) > 0) {
					$report[] = "\tFault Code info:".$nodelist->item($i)->getAttribute('comment').":";
				}
			}
		}
	}

	// Pull out the testwithstate blocks, ah such fun
	if ($print_test) {
		$nodelist = $dom->getElementsByTagName('testwithstate');
		if ($nodelist->length == 0){
			echo "no testwithstate tags found, is this a troubleshooting track?";
		} else {
			for ($i = 0; $i < $nodelist->length; $i++) {
				$testID = $nodelist->item($i)->getAttribute('id');
			
				$vars = $nodelist->item($i)->getElementsByTagName('variable');
				$varRef = $nodelist->item($i)->getElementsByTagName('variableref');
			$diagnostics = $nodelist->item($i)->getElementsByTagName('diagnostic_group');
				
				// add in the variables to the array first. Set them to false for if a match has been found or not
				for ($j = 0; $j < $vars->length; $j++) {
					$testIDArray[$testID][$vars->item($j)->getAttribute('name')]  = "Variable only";
				}
				// now grab out the variables for each block and again, set to false for having found a match yet.
				for ($j = 0; $j < $vars->length; $j++) {
					if (is_object($varRef->item($j))) {
						if (!(isset( $testIDArray[$testID][$varRef->item($j)->getAttribute('name')]) )) {
							$testIDArray[$testID][$varRef->item($j)->getAttribute('name')] = "Variableref only";
						} elseif (strcasecmp( $testIDArray[$testID][$varRef->item($j)->getAttribute('name')], "Variable only") == 0) {
							$testIDArray[$testID][$varRef->item($j)->getAttribute('name')] = "match";
						}
					} else {
						$report[] = "Check for duplicate diagnotic_initial tag in testwithstateid:".$testID;
					}
				}
			}
		}
	}

	// now check out the links to make sure they all have a prompt child
	if ($print_test) {
		$nodelist = $dom->getElementsByTagName('link');
		if ($nodelist->length == 0){
			$report[] = "\tNo link tags found in file";
			if ($htmlFlag) {
			$report[] = "<b>".array_pop($report)."</b>";
			} 
		} else {
			foreach ($nodelist as $node) {
				$curLink = $node->getAttribute('xlink:href');
				if ($debug) {
					$report[] = "\tChecking link ".$curLink.":";
					if ($htmlFlag) {
						$report[] = "<b>".array_pop($report)."</b>";
					} 
				}
				if (strpos($curLink, $wpno) !== false) {
					// get the substring for the part after the #, if the pound sign exists, then check and see if the sublink is in this file
					if (strpos($curLink, "#")  !== false) {
						$internalLink = substr($curLink, strpos($curLink,"#")+1);
						$intLinkArray[] = $internalLink;
						if (!($debug)) {
								$report[] = "\tChecking link ".$curLink;
								if ($htmlFlag) {
									$report[] = "<p><b>".array_pop($report)."</b>";
								} 
						}
						if (array_key_exists($internalLink, $testIDArray)) {
							$report[] = "\t\tfound testwithstate id match for ".$internalLink;
						} else {
							$report[] = "\t\t<h2>Warning: no testwithstate id match found for ".$internalLink."</h2>";
						}
						if ($htmlFlag) {
							$report[] = array_pop($report)."</p>";
						} 
					}
				} else {
					if ($debug) {
						$report[] = "\t\tExternal link, not checking for testwithstate link";
						if ($htmlFlag) {
							$report[] = array_pop($report)."</p>";
						} 
					}
				}
			
				if ( strcmp($node->getAttribute('xreftype'),'test') == 0) {
					if ($debug) {
						$report[] = "\t\tPrompt tag N/A";
					}
				} else {
					foreach($node->childNodes as $child) {
						if (strcasecmp($child->nodeName,'prompt') == 0)	{
							if ($debug) {
								$report[] = "\t\tPrompt tag Exists";
							}
						} elseif (strcmp($child->nodeName,'#text') != 0) {
							if ($debug) {
								$report[] = "\t\t<h2>Warning: Check Link for prompt tag: </h2>";
							}
						}
					}
				}
				$linksCntr++;
			}
		}
	}
	$report[] = "\r\n";
	
	if ($debug) {
		var_dump($intLinkArray);
		var_dump($testIDArray);
	}

		// dig out a list of the diagnostic tags so a visual check can be done on the mfg and model
	if ($print_parms) {
		$nodelist = $dom->getElementsByTagName('diagnostic');
		if ($nodelist->length == 0){
			$report[] = "\tNo diagnostic tags found in file";
		} else {
			foreach ($nodelist as $node) {
				$curMFR = $node->getAttribute('mfr');
				$curModel = $node->getAttribute('model');
				$curProtocol = $node->getAttribute('protocol');
				$curTestname = $node->getAttribute('testname');
				$report[] = "\tDiagnostic values: mfr:".$node->getAttribute('mfr').":model:".$node->getAttribute('model').":protocol:".$node->getAttribute('protocol').":testname:".$node->getAttribute('testname').":";
				if ($htmlFlag) {
					$report[] = array_pop($report)."";
				} 
				$parmlist = $node->getElementsByTagName('sendparameter');
				foreach ($parmlist as $parm) {
					$report[] = "\t\tDiagnostic Parameters";
					foreach ($parm->childNodes as $child) {
						$report[] = "\t\t\t:".trim($child->nodeValue).":";
						// echo "child nodes:".$child->nodeValue."\r\n";
					}
				}

			}
		}
	}
	
	if ($print_test) {
		foreach ($testIDArray as $tsKey => $varCheck) {
			$report[] = "\ttestwithstate id: ".$tsKey." status";
			if ($htmlFlag) {
				$report[] = "<b>".array_pop($report)."</b>";
			} 
			if ( in_array($tsKey, $intLinkArray ) ) {
				$report[] = "\t\tBlock has link";
			} else {
				$report[] = "\t\t<h2>Warning: block has no internal links to it, orphan or first block</h2>";
			}
		
			$cntr = 0;
			foreach ($varCheck as $key => $value) {
				if (strcmp($value,'match') != 0) {
					$report[] = "\t\t\t<h2>Warning: ".$value.": \t:".$key.":</h2>";
					$cntr++;
				}
			}
			if ($cntr == 0) {
				$report[] = "\t\tVariables fine";
			}
		}
		$report[] = "\r\n";
	}	
} 
if ($htmlFlag) {
	$report[] = "<h3><a href='index.php'>Click to check more files</a></h3>";
	$report[] = "</body>\r\n</html>";
}
for ($i=0;$i < count($report) ;$i++) {
	if ($htmlFlag) {
		echo $report[$i]."<br>\r\n";
	} else {
		echo $report[$i]."\r\n";
	}
} 


// Function used to clean out stuff that DOM XML parser is not fond of, things like DTD reference, entities and such.
function clean_xml($filename){

/*	print "\r\n";
    var_dump($filename);
	print "\r\n"; */
	$document = file_get_contents($filename, 0, NULL, 0, 900000);

	$search = array(	"'&(.|\n)+?;'i",		// replace html entities
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$document); 

	$search = array(	"'\<\!\-\-ArborText\, Inc\.\, +.* v\.4002\-\-\>'i",
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$text);	

	$search = array(	"'(\<\!ENTITY.*?>)'i",
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$text,75,$count);	

	// print "dumping text left over...\r\n";
	// var_dump($text);

	return $text;

}

function scanDirectory($rootDir, $allData=array()) {
global $ext;
    // set filenames invisible if you want
    $invisibleFileNames = array(".", "..", ".htaccess", ".htpasswd", ".svn");
    // run through content of root directory
    $dirContent = scandir($rootDir);
    foreach($dirContent as $key => $content) {
        // filter all files not accessible
        $path = $rootDir.'/'.$content;
        if(!in_array($content, $invisibleFileNames)) {
            if(is_file($path) && stristr($path, $ext) && !stristr($path, ".sgml")&& is_readable($path)) {
                $allData[] = $path;
            }elseif(is_dir($path) && is_readable($path)) {
                $allData = scanDirectory($path, $allData);
            }
        }
    }
    return $allData;
}
?>
