<?php 
// +----------------------------------------------------------------------+
// | EMS NG Troubleshooting Checker scripts                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2013 VDL-5 Technologies Inc.                           |
// |                                                                      |
// +----------------------------------------------------------------------+
// | Authors: Tamara S Harpster <harpstert@vdl5tech.com                   |
// +----------------------------------------------------------------------+
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

include 'common.inc';
include 'config.inc';

// Set defaults for options in case they don't get set. Only exception is setting the file location for the replacement directives, have to have that.
$indir="C:\IETM\M915_IETM\T - Troubleshooting procedures\\";
$logfile="ts_check.log";
$ext=".xml";
$debug = false;
$all_files = array();
$clean_files = array();
$htmlFlag = false;
// Set up an Array to store the messages that are found during the checks
$report = array();
$print_test = true;
$print_link = false;
$print_parms = false;

if ($argc > 1) {
	// script is being run from command line so process data accordingly
	// Remember to finish up the File argument at some point in the future
	// Process the argument, set some defaults for directories if they don't get set. 
	for ($i=1;$i<$argc;$i++){
		switch ($argv[$i]) {
			case "-indir":
				$i++;
				$indir= (file_exists(addslashes($argv[$i]))) ? addslashes($argv[$i]) : "\\.";
				$all_files = scanDirectory($indir);
				$clean_files = str_ireplace($indir, "" , $all_files);
				break;
			case "-debug":
				$i++;
				$debug= addslashes($argv[$i]);
				break;
			case "-list":
				$i++;
				$fileList = (file_exists(addslashes($argv[$i]))) ? addslashes($argv[$i]) : "in_files.txt";
				$clean_files = file($fileList, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
				break;
			case "-log":
				$i++;
				if (!(file_exists(addslashes($argv[$i])))) {
					mkdir(addslashes($argv[$i]),0777, true);
				}
				$logfile= addslashes($argv[$i]);
				break;
			case "-file":
				$i++;
				print addslashes($argv[$i])."\r\n";
				if (file_exists(addslashes($argv[$i])) ){
					$lines_temp=file(addslashes($argv[$i]), FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
					$lines2_temp=implode($lines_temp);
					$lines = split($endToken,$lines2_temp);
					// added in a pop in order to clean off a blank being read in for the file, shrug, it works, and gets rids of the preg_replace errors so I'm happy
					$tmplines = array_pop($lines);
				} else {
					print "nope, sorry, no file";
					exit;
				}
				break;
			default:
		}
	}
} else {
	// calling this guy from a web page so set a few items here for use later on
	$htmlFlag = true;
	$clean_files = array_keys( $_GET, "on");
	$indir = $base_dir.$ietms[$_GET['inputdir']]["dir"]."\\T - Troubleshooting procedures\\";
	$report[] = "<html>\r\n<head>\r\n<title>EMS NG Troubleshooting WP Checker</title></head>\r\n<body style='font-family: Arial' scroll='auto'>";
}

	// read in and build array for lookup of Titles for EMS NG tracks
	$lines_temp=file("C:\\IETM\\HMEEI_IETM\\hmee_ng_titles.txt", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
	$lookUp = array();
	for ($i=0; $i < count($lines_temp);$i++) {
		$lookUp[strtok($lines_temp[$i],"\t")] = substr($lines_temp[$i],strpos($lines_temp[$i],"\t")+1);
		// print ("token:".strtok($lines_temp[$i],"\t").":tab location:".strpos($lines_temp[$i],'\t').":second:".substr($lines_temp[$i],strpos($lines_temp[$i],"\t")+1)."<br>\r\n");
	}
	 // var_dump($lookUp);
	
if ($htmlFlag) {
	$tempstr = "";
	for ($k=0;$k< sizeof($clean_files); $k++) {
		$tempstr = $tempstr."&".$clean_files[$k]."=on";
	}

	$report[] = "<ul>
					<li><a href='track_report.php?inputdir=".$_GET['inputdir'].$tempstr."'>Click to check the same files</a></li>
					<li><a href='trackflow.php?inputdir=".$_GET['inputdir']."'>Click to check other files in this IETM</a></li>
					<li><a href='index.php'>Click to files in another IETM</a></li>
					</ul>";
}
// Dump out an index of the files being checked
if ($htmlFlag) {
	foreach ($clean_files as $file_name){ 
		$report[] = "<a href='#".trim($file_name)."'>".trim($file_name)." results</a><br>";
	}
}

foreach ($clean_files as $file_name){ 
	$clean_text = clean_xml($indir.trim($file_name).".xml");
	// var_dump($clean_text);

	$report[] = "<a name='".trim($file_name)."' STYLE='font-family:sans-serif; font-size:120%; font-weight:bold;'>".$lookUp[trim($file_name)]."</a>";
	$report[] = "<br><b>File name:".trim($file_name)."</b>";
 
	
	// do some initialization here of items that are needed internally to the program
	// Stores the links that have been dug out of the file along with results of various check on the links.
	$linksArray = array();
	$linksCntr = 0;
	// Keeps track of the testwithstate ids for the file so they can be used to check the file links
	$testIDArray = array();
	$ptrArray = array();
	$treeOut = array();
	
	// Setup array to store internal links for checks against Testwithstates
	$intLinkArray = array();

	$flag = preg_match_all  ( "(wpno\=\"T[0-9A-Z]+?\")i", $clean_text  , $matches );
	$wpno = substr($matches[0][0], stripos($matches[0][0], '"' )+1, (strlen($matches[0][0]) - stripos($matches[0][0], '"' )) - 2);
	// print "wpno:".$wpno.":strlen:". strlen($matches[0][0]).":pos:".stripos($matches[0][0], '"' )."\r\n<br>";

	$dom = new DomDocument();
	$dom->loadXML($clean_text);

	// Pull out the testwithstate blocks, ah such fun
	if ($print_test) {
		$nodelist = $dom->getElementsByTagName('testwithstate');
		if ($nodelist->length == 0){
			echo "no testwithstate tags found, is this a troubleshooting track?";
		} else {
			$startPtr = $nodelist->item(0)->getAttribute('id');	
			for ($i = 0; $i < $nodelist->length; $i++) {
				$testID = $nodelist->item($i)->getAttribute('id');	
				$titles  = $nodelist->item($i)->getElementsByTagName('title');
				$testIDArray[$testID]['title'] = $titles->item(0)->nodeValue;
				$choice = $nodelist->item($i)->getElementsByTagName('choice');
				
				// This is getting the choice items, setting up the array for fillin  with the links from the evaluate session
				for ($j = 0; $j < $choice->length; $j++) {
					$button = $choice->item($j)->getElementsByTagName('text')->item(0)->nodeValue;
					$varName = $choice->item($j)->getElementsByTagName('statemanipulation')->item(0)->getElementsByTagName('variable')->item(0)->getAttribute('name');
					
					$testIDArray[$testID][$varName]['button'] = $button;
				}
				
				// now pull out the evalute section and match up with the choice section to start building the items up
				$evalNode = $nodelist->item($i)->getElementsByTagName('evaluate');
				$varRefNode = $evalNode->item(0)->getElementsByTagName('expression');
				$resultBlock = $evalNode->item(0)->getElementsByTagName('resultwithstate');
				
				for ($j = 0; $j < $resultBlock->length; $j++) {
					$varBlock = $varRefNode->item($j)->getElementsByTagName('variableref');
					$varRef = $varBlock->item(0)->getAttribute('name');
					$linkBlock = $resultBlock->item($j)->getElementsByTagName('link');
					for ($k = 0; $k < $linkBlock->length; $k++) {
						$linkTemp = substr($linkBlock->item($k)->getAttribute('xlink:href'), 7);
						// Get the link for printing purposes
						$testIDArray[$testID][$varRef]['link'][$linkTemp] = NULL;
						
						$promptList = $linkBlock->item($k)->getElementsByTagName('prompt');
						if ($promptList->length !=0) {
							$testIDArray[$testID][$varRef]['link'][$linkTemp] = $promptList->item(0)->nodeValue;
							// print "prompt value:".$promptList->item(0)->nodeValue.":<br>";
						}
						$linkWP = substr($linkTemp, 7, stripos($linkTemp, "#" )-(stripos($linkTemp, "IETM://" )+7));
						// print ("link string:".$linkTemp.": start pos:".(stripos($linkTemp, "IETM://" )+7).": end pos".stripos($linkTemp, "#" ).": partial:".$linkWP."\r\n<br>");
		
						//build up the pointers to use when building the tree
						$matches = array();
						// print "current record:".$wpno."#".$testID."/".$linkTemp."/\r\n<br>";
						
						if ((stripos(implode(";", $ptrArray),$wpno."#".$testID.";") === FALSE ) && (empty($ptrArray))){
							$ptrArray[$wpno."#".$testID] = $wpno."#".$testID;
							// print "adding in new record...<br>\r\n";
						}
						
						if (preg_match(";[A-Z#0-9/]{8,}".$wpno."#".$testID."/;" , implode(";",$ptrArray) , $matches) > 0 ) {
							// print "found a match...<br>\r\n";
							$ptrArray[$matches[0].$linkTemp."/"] = $matches[0].$linkTemp."/";
						} else {
							// print "subrec added...<br>\r\n";
							$ptrArray[$wpno."#".$testID."/".$linkTemp."/"] = $wpno."#".$testID."/".$linkTemp."/";
						}				
					}
				}
			}
			// var_dump($ptrArray);
			$dumpArray = array();
			$dumpArray = explodeTree($ptrArray, "/", true);
			 // var_dump($dumpArray);
			
			plotTree($dumpArray);
			for ($k=0;$k< sizeof($treeOut);$k++)
			{
				$report[] = $treeOut[$k];
			}
			// var_dump($treeOut);
			
			foreach ($testIDArray as $tsKey => $varCheck) {
				$report[] = "--------------------------------------------------------";
				$report[] = "<strong>".$tsKey.":".$varCheck['title']."</strong>";
		
				// var_dump($varCheck);			
				$report[] = "<table border='1'>";
				foreach ($varCheck as $varRefer => $buttonArray) {
					if (is_array($buttonArray)) {
						$report[] = "<tr><td STYLE='font-family:sans-serif; font-style:italic; background:#1E55A0 none; color:#000; width:15em'>".rtrim($buttonArray['button'])."</td><td STYLE='font-family:sans-serif; background:#66cc66 none; color:#000; width:25em'>";
						// var_dump($buttonArray);

						// .(stripos($promptKey,"#")===FALSE) ? $lookUp[$promptKey] : $lookUp[substr($promptKey,0,stripos($promptKey,"#"))].
						foreach ($buttonArray['link'] as $promptKey => $promptText) {
							$greenBar = "";
							if (strlen($promptText) > 0 ) {
								$greenBar = "<strong>Green Bar</strong>: ".$promptText."<br>:";
							} 
							if ( stripos($promptKey,"#") === FALSE) {
								$report[] = preg_replace("'([\r\n])[\s]+'"," ",$greenBar)."\n<li><a href=''>".$lookUp[$promptKey]."<br>".$promptKey."</li>";
							} else {
								$report[] = preg_replace("'([\r\n])[\s]+'"," ",$greenBar)."\n"."<li><a href=''>".$lookUp[substr($promptKey,0,stripos($promptKey,"#"))]."<br>".$promptKey."</li>";
							}


						}
						$report[]="</td></tr>\r\n";
					}
				}
				$report[] = "</table>\r\n";
				 $report[] = "<br><br><br><br><br><br>\r\n\r\n\r\n\r\n\r\n";
			}
		}
	}

	$report[] = "\r\n";
	
	if ($debug) {
		var_dump($testIDArray);
	}
	
} 
if ($htmlFlag) {
	$report[] = "<h3><a href='index.php'>Click to check more files</a></h3>";
	$report[] = "</body>\r\n</html>";
}

for ($i=0;$i < count($report) ;$i++) {
	if ($htmlFlag) {
		echo $report[$i]."<br>\r\n";
	} else {
		echo $report[$i]."\r\n";
	}
} 
// Function used to clean out stuff that DOM XML parser is not fond of, things like DTD reference, entities and such.
function clean_xml($filename){

	$document = file_get_contents($filename, 0, NULL, 0, 200000);
	 // var_dump($document);
	$search = array(	"'&(.|\n)+?;'i",		// replace html entities
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$document); 

	$search = array(	"'\<\!\-\-ArborText\, Inc\.\, +.* v\.4002\-\-\>'i",
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$text);	
	
	$search = array(	"'(\<\!ENTITY.*?>)'i",
						);
	$replace = array( 	"",
						);
	$text = preg_replace($search,$replace,$text,200,$count);	

	
	 // var_dump($text);
	
	return $text;

}

function scanDirectory($rootDir, $allData=array()) {
global $ext;
    // set filenames invisible if you want
    $invisibleFileNames = array(".", "..", ".htaccess", ".htpasswd", ".svn");
    // run through content of root directory
    $dirContent = scandir($rootDir);
    foreach($dirContent as $key => $content) {
        // filter all files not accessible
        $path = $rootDir.'/'.$content;
        if(!in_array($content, $invisibleFileNames)) {
            // if content is file & readable, add to array
            if(is_file($path) && stristr($path, $ext) && !stristr($path, ".sgml")&& is_readable($path)) {
                // save file name with path
                $allData[] = $path;
            // if content is a directory and readable, add path and name
            }elseif(is_dir($path) && is_readable($path)) {
                // recursive callback to open new directory
                $allData = scanDirectory($path, $allData);
            }
        }
    }
    return $allData;
} 

/**
 * Explode any single-dimensional array into a full blown tree structure,
 * based on the delimiters found in it's keys.
 *
 * @author  Kevin van Zonneveld <kevin@vanzonneveld.net>
 * @author  Lachlan Donald
 * @author  Takkie
 * @copyright 2008 Kevin van Zonneveld (http://kevin.vanzonneveld.net)
 * @license   http://www.opensource.org/licenses/bsd-license.php New BSD Licence
 * @version   SVN: Release: $Id: explodeTree.inc.php 89 2008-09-05 20:52:48Z kevin $
 * @link    http://kevin.vanzonneveld.net/
 *
 * @param array   $array
 * @param string  $delimiter
 * @param boolean $baseval
 *
 * @return array
 */
function explodeTree($array, $delimiter = '_', $baseval = false)
{
  if(!is_array($array)) return false;
  $splitRE   = '/' . preg_quote($delimiter, '/') . '/';
  $returnArr = array();
  foreach ($array as $key => $val) {
    // Get parent parts and the current leaf
    $parts  = preg_split($splitRE, $key, -1, PREG_SPLIT_NO_EMPTY);
    $leafPart = array_pop($parts);
 
    // Build parent structure
    // Might be slow for really deep and large structures
    $parentArr = &$returnArr;
    foreach ($parts as $part) {
      if (!isset($parentArr[$part])) {
        $parentArr[$part] = array();
      } elseif (!is_array($parentArr[$part])) {
        if ($baseval) {
          $parentArr[$part] = array('__base_val' => $parentArr[$part]);
        } else {
          $parentArr[$part] = array();
        }
      }
      $parentArr = &$parentArr[$part];
    }
 
    // Add the final part to the structure
    if (empty($parentArr[$leafPart])) {
      $parentArr[$leafPart] = $val;
    } elseif ($baseval && is_array($parentArr[$leafPart])) {
      $parentArr[$leafPart]['__base_val'] = $val;
    }
  }
  return $returnArr;
}

function plotTree($arr, $indent=0, $mother_run=true){
global $treeOut, $wpno;
 
    foreach($arr as $k=>$v){
        // skip the baseval thingy. Not a real node.
        if($k == "__base_val") continue;
        // determine the real value of this node.
        $show_val = ( is_array($v) ? $v["__base_val"] : $v );
 
        // show the indents
		$tempstr = str_repeat("==", $indent);
        if($indent == 0){
            // this is a root node. no parents
            $tempstr = $tempstr."Start: ==>";
        } elseif(is_array($v)){
            // this is a normal node. parents and children
           $tempstr = $tempstr."> ";
        } else{
            // this is a leaf node. no children
			$tempstr = $tempstr."| ";
        }
 
        // show the actual node
		if (stripos($k, $wpno) === FALSE){
			// print "No find of wpno".$k."wpno:".$wpno."\r\n";
			$linky = $k;
		} else {
			$linky = substr($k, stripos($k, "#") + 1);
			// print "found the wpno\r\n";
		}
        $treeOut[] = $tempstr.$linky;
 
        if(is_array($v)){
            // this is what makes it recursive, rerun for childs
            plotTree($v, ($indent+1), false);
        }
    }
 
}
?>
